#!/usr/bin/ruby
# encoding: utf-8

require 'optparse'
require 'ostruct'

$:.unshift File.dirname(__FILE__)
$:.unshift File.join(File.dirname(__FILE__), "lib")
$:.unshift File.join(File.dirname(__FILE__), "vendor")

HOME = File.expand_path("~")
AGVERSION = "1.1.5"
AGVERSION_DATE = "December 1, 2014"

require 'log'
require 'file_controller'
require 'op4_report_controller'
require 'op3_report_controller'
require 'basetest'

# Vendor
require 'json'

# Libraries
require 'osx'
require 'op4'
require 'op3'
require 'safari'
require 'chrome'
require 'firefox'
require 'opera'
require 'helpers'
require 'ubiquity'
require 'dropbox'

class AppController

	def initialize
		@working_folder = File.expand_path "~/Library/Application\ Support/1Password\ 4/Diagnostics" 
		@reportFolder = generateReportFolderLocation
		@options = OptionParser.parse(ARGV)

		clean_up
		create_report_folder
	end

	def main
		# Configure around where to log
		if @options.log
			Log.instance.change_file("#{@reportFolder}/agiletools-log.txt")
			Log.instance.info("[Logger] Initialized: agiletools-log.txt")
		else
			Log.instance.info("[Logger] Initialized: STDOUT")
		end
	
		Log.instance.info("[Diagnostics] Version: #{AGVERSION}")

		begin
			gather_data
		rescue StandardError => e
			Log.instance.warn("[Diagnostics] Exception in OP4/5 Diagnostics!")
			Log.instance.warn("[Diagnostics] Backtrace \n\n #{e.backtrace.join("\n")}\n\n")
			Log.instance.warn("[Diagnostics] Message: #{e.message}")
			Log.instance.warn("[Diagnostics] Inspect: #{e.inspect}")
		end		

		if @options.report
			generate_report(@options.safemode)
		end
	end

	# Generate the name of the folder and path for the report
	# Example: 1PasswordDiagnostics-Apollo-2013-08-30_11-30
	#
	# Returns (String) full path to report folder
	def generateReportFolderLocation
		hostname = `hostname -s`.chomp	#chomp off \n
		date = Time.now.strftime("%Y-%m-%d_%H-%M")
		@reportFoldername = "1PasswordDiagnostics-#{hostname}-#{date}"
		reportFolder = File.expand_path "~/Library/Application\ Support/1Password\ 4/Diagnostics/#{@reportFoldername}/"
		return reportFolder
	end

	def script_resource(folder_name)
		return File.join(File.dirname(__FILE__), folder_name)
	end

	def generate_report(safemode)
		begin
			@data.logs = FileController.new(@reportFolder)
		rescue StandardError => e
			Log.instance.warn("[Diagnostics] Exception in File Controller!")
			Log.instance.warn("[Diagnostics] Backtrace \n\n #{e.backtrace.join("\n")}\n\n")
			Log.instance.warn("[Diagnostics] Message: #{e.message}")
		end			
				
		begin
			# OnePassword 4
			template = script_resource("op4_template.html.erb")
			output_file = "#{@reportFolder}/1Password4.html"
			op4_controller = OP4ReportController.new(@data, output_file, template, "OP4", safemode)
		rescue StandardError => e
			Log.instance.warn("[Diagnostics] Exception in OP4/5 Diagnostics!")
			Log.instance.warn("[Diagnostics] Backtrace \n\n #{e.backtrace.join("\n")}\n\n")
			Log.instance.warn("[Diagnostics] Message: #{e.message}")
			Log.instance.warn("[Diagnostics] Inspect: #{e.inspect}")
		end			

		begin
			# OnePassword 3
			op3_template = script_resource("op3_template.html.erb")
			op3_output_file = "#{@reportFolder}/1Password3.html"
			op3_controller = OP3ReportController.new(@data, op3_output_file, op3_template, "OP3", safemode)
		rescue StandardError => e
			Log.instance.warn("[Diagnostics] Exception in OP3 Diagnostics!")
			Log.instance.warn("[Diagnostics] Backtrace \n\n #{e.backtrace.join("\n")}\n\n")
			Log.instance.warn("[Diagnostics] Message: #{e.message}")
			Log.instance.warn("[Diagnostics] Inspect: #{e.inspect}")
		end

		copy_html_assets

		`open '#{output_file}'` if ENV['OPDIAGS_AUTOMATICALLY_OPEN']

		Log.instance.info("[Diagnostics] Saving Zip File to: #{@reportFolder}.zip")
		Log.instance.info("[Diagnostics] Closing Log")
		
		if @options.log
			Log.instance.close
		end

		zip_report

	end

	# Gathers all of the report data.
	#		Every subsection will need to be put here to generate data
	def gather_data
		@data = OpenStruct.new

		Log.instance.info("[OP] Gathering OS X data")
		@osx = OSX.new
		@data.osx = @osx

		Log.instance.info("[OP] Gathering OP4 data")		
		@op4 = OP4.new(@safemode)
		@data.op4 = @op4

		Log.instance.info("[OP] Gathering OP3 data")
		@op3 = OP3.new(@safemode)
		@data.op3 = @op3

		Log.instance.info("[OP] Gathering Safari data")		
		@safari = Safari.new
		@data.safari = @safari

		Log.instance.info("[OP] Gathering Firefox data")		
		@firefox = Firefox.new
		@data.firefox = @firefox

		Log.instance.info("[OP] Gathering Ubiquity data")		
		@ubiquity = Ubiquity.new
		@data.ubiquity = @ubiquity

		Log.instance.info("[OP] Gathering Chrome data")		
		@chrome = Chrome.new
		@data.chrome = @chrome

		Log.instance.info("[OP] Gathering Dropbox data")		
		@dropbox = Dropbox.new
		@data.dropbox = @dropbox

		Log.instance.info("[OP] Gathering Opera data")
		@opera = Opera.new
		@data.opera = @opera
	end

	def copy_html_assets
		FileUtils.cp_r(script_resource("css"), @reportFolder)
		FileUtils.cp_r(script_resource("img"), @reportFolder)
		FileUtils.cp_r(script_resource("js"), @reportFolder)
		FileUtils.cp_r(script_resource("fonts"), @reportFolder)
	end

	def zip_report
		# Zip ~/Library/Application Support/1Password 4/Diagnostics/1PasswordDiagnostics-Hostname-Date/
		pushd_command = "pushd \'#{@working_folder}\'"
		zip_command = "zip -r '#{@reportFolder}.zip' '#{@reportFoldername}'"
		puts "#{@reportFolder}.zip"
		zip = `#{pushd_command} ; #{zip_command} ; popd` 
	end

	def clean_up
		FileUtils.rm_rf("#{@working_folder}")
		
		FileUtils.mkdir_p(@working_folder)
	end

	def create_report_folder
		Dir.mkdir(@reportFolder)
	end
end

class OptionParser 
	def self.parse(args)
		options = OpenStruct.new
		options.report = true
		options.safemode = false
		options.log = true
		options.op3 = true 

		opts = OptionParser.new do |opts|

			opts.program_name = "AgileTools"
			opts.version = AGVERSION 
			opts.banner = "Usage: agiletools.rb [options]"

			opts.separator ""
			opts.separator "One of the following options:"

			opts.on("-s", "--safemode", "Safe Mode reducing options for problematic functions. i.e. mdfind") do |s|
				options.safemode = true
			end

			opts.on("--skip-op3", "Do not generate a 1Password 3 Diagnostics Report") do |op3|
				options.op3 = false 
			end

			opts.on("--nolog", "Don't log to file, instead output to STDOUT") do |nolog|
				options.log = false
			end

			opts.separator ""

			opts.on_tail("-v", "--version", "Show version") do 
				puts opts.program_name + " " + opts.version
				exit
			end

			opts.on_tail("-h", "--help", "Show this message") do
				puts opts
				exit
			end

		end # opts

		opts.parse!(args)
		options

	end # parse()
end

# Launch main function
if __FILE__ == $PROGRAM_NAME
	app = AppController.new
	app.main
end
