require 'ostruct'
require 'helpers'

class Firefox
	attr_reader :version
	attr_reader :profiles
	attr_reader :jse3
	attr_reader :valid_code_signature
	attr_reader :code_signature_output

	def initialize
		@version = get_version
		@profiles = get_profiles
		@jse3 = get_jse3_database_info
		
		@valid_code_signature = Helpers.verify_code_signature("/Applications/Firefox.app/")
		@code_signature_output = Helpers.code_signature_output("/Applications/Firefox.app/")
	end

	def get_version
		Helpers.findPlistValueForKey("/Applications/Firefox.app/Contents/Info.plist", "CFBundleShortVersionString")
	end

	def get_profiles
		begin
			profile_ini_path = "#{HOME}/Library/Application Support/Firefox/Profiles.ini"
			
			return [] unless File.exists?(profile_ini_path)

			profile_file = File.open(profile_ini_path)
			lines = profile_file.read

			profile_ini_contents = lines
      
      result = ""
      profile_list = []
      current = {}
      lines.split("\n").each { |l|
        l = l.strip
        result = "#{result}\n    #{l.inspect}"
        
        if l =~ /^\[(.*)\]$/
          profile_list << current unless current.empty?
          current = {}
        end
        
        unless current.nil?
          current[:name] = $1 if l =~ /^Name=(.*)/
          current[:path] = $1 if l =~ /^Path=(.*)/
          current[:relative] = $1 == "1" if l =~ /^IsRelative=(.*)/
          current[:default] = $1 if l =~ /^Default=(.*)/
        end
      }
      profile_list << current unless current.nil?
    rescue
      print("Error occured while reading the Profiles.ini file: #{$!}")
    end

		profiles = []

		if profile_list != nil
			profile_list.each do |profile|
				temp_profile = OpenStruct.new
				temp_profile.name = profile[:name]
				temp_profile.path = profile[:path]
				temp_profile.relative = profile[:relative]
				temp_profile.websocket_enabled = websocketEnabled?("#{Helpers.home}/Library/Application Support/Firefox/#{profile[:path]}")
				temp_profile.javascript_enabled = javascriptEnabled?("#{Helpers.home}/Library/Application Support/Firefox/#{profile[:path]}") 
				temp_profile.proxy_type = proxyType("#{Helpers.home}/Library/Application Support/Firefox/#{profile[:path]}")

				temp_profile.installed_extensions = installedExtensions("#{HOME}/Library/Application Support/Firefox/#{profile[:path]}")

				profiles.push temp_profile
			end
		end


		return profiles
	end

	def installedExtensions_pre29(profile_path)

		extensions = []
		if File.exists?("#{profile_path}/extensions.sqlite")
			result = `sqlite3 "#{profile_path}/extensions.sqlite" "SELECT id,version,userDisabled FROM addon;"`

			return extensions if result.empty?

			lines = result.split("\n")

			lines.each do |extension|
				items = extension.split("|")
				current = OpenStruct.new
				current.name = items[0]
				current.version = items[1]
				if items[2] == "1"
					current.enabled = false
				else
					current.enabled = true
				end

				extensions.push current
			end
		end

		return extensions
	end

	def installedExtensions(profile_path)
		json_file = "#{profile_path}/extensions.json"
		return [] unless File.exists?(json_file)

		json_data = File.read(json_file)
		parsed_json = JSON.parse json_data

		extensions = []

		parsed_json["addons"].each do |addon|
			current = OpenStruct.new
			current.name = addon["defaultLocale"]["name"]
			current.version = addon["version"]
			if addon["userDisabled"] == true
				current.enabled = false
			else
				current.enabled = true
			end
			current.appDisabled = addon["appDisabled"]

			extensions.push current
		end

		return extensions
	end

	def javascriptEnabled?(profile_path)
		if File.exists?("#{profile_path}/prefs.js")
			js = `grep javascript\.enabled "#{profile_path}/prefs.js"`
			if js =~ /false/
				return false
			end
		end

		return true
	end

	def websocketEnabled?(profile_path)
		if File.exists?("#{profile_path}/prefs.js")
			ws = `grep websocket\.enabled "#{profile_path}/prefs.js"`
			if ws =~ /false/
				return false
			end
		end
		
		return true 
	end

	def defaultProfile
		profiles = self.profiles
	
		# Return profile 0 if there is only one
		return 0 if profiles.count == 1

		profiles.each_with_index do |profile, count|
			if profile[:default]
				return count
			end
		end
	end

	def get_jse3_database_info
		jse3_info = OpenStruct.new
		if File.exists?("#{HOME}/Library/Application\ Support/Firefox/Profiles/OnePassword.sqlite")	
			jse3_info.schema = get_jse3_schema
			jse3_info.config = get_jse3_config
			jse3_info.total_items = get_jse3_total_items
			jse3_info.total_keys = get_jse3_total_keys
			jse3_info.unsynced_items = get_jse3_unsynced_items
		else
			jse3_info.schema = "DB Does Not Exist"
			jse3_info.config = "DB Does Not Exist"
			jse3_info.total_items = 0 
			jse3_info.total_keys = 0
			jse3_info.unsynced_items = 0
		end

		return jse3_info

	end

	def get_jse3_schema
		path = File.expand_path("~/Library/Application\ Support/Firefox/Profiles/OnePassword.sqlite")
		schema = Helpers.sqlite_query(path, ".schema")

		return schema
	end

	def get_jse3_config
		path = File.expand_path("~/Library/Application\ Support/Firefox/Profiles/OnePassword.sqlite")
		config = Helpers.sqlite_query(path, "SELECT * FROM config;")

		return config
	end

	def get_jse3_total_items
		path = File.expand_path("~/Library/Application\ Support/Firefox/Profiles/OnePassword.sqlite")
		raw_total_items = Helpers.sqlite_query(path, "SELECT count(*) FROM items;")

		return raw_total_items.chomp!.to_i
	end

	def get_jse3_total_keys
		path = File.expand_path("~/Library/Application\ Support/Firefox/Profiles/OnePassword.sqlite")
		raw_total_keys = Helpers.sqlite_query(path, "SELECT count(*) FROM keys;")

		return raw_total_keys.chomp!.to_i
	end

	def get_jse3_unsynced_items
		path = File.expand_path("~/Library/Application\ Support/Firefox/Profiles/OnePassword.sqlite")
		raw_unsynced_items = Helpers.sqlite_query(path, "SELECT count(*) FROM items WHERE contents_hash = \"777\";")

		return raw_unsynced_items.chomp!.to_i
	end

	def crashReports
		# Is this even needed? 
	end

	def proxyType(profile_path)
		# Default is "5" because defaults are not set in prefs.js
		#		If not set, it's "5"
		type = "5" 

		# Never trust the path sent in
		if File.exists?("#{profile_path}/prefs.js")
			file = File.read("#{profile_path}/prefs.js")
			file.each_line do |line|
				# network.proxy.type tells us what type of proxy is set
				begin
					type = $1 if line =~ /^user_pref\(\"network\.proxy\.type\", (\d)\);/
				rescue ArgumentError => error
					Log.instance.warn("[Firefox] Non-ASCII characters found in prefs.js")
					type = "99"
				end
			end
		end

		# These match the wording in the app
		case type
		when "0" 
			return "No Proxy"
		when "1"
			return "Manual Proxy Configuration"
		when "2"
			return "Automatic proxy configuration URL"
		when "4"
			return "Auto-detect proxy settings for network"
		when "5"
			return "Use system proxy settings"
		when "99"
			return "Non-Ascii Characters found, cannot accurately determine Proxy Type"
		end
	end

	def appSupportListing
		return Helpers.ls_output_for_path("~/Library/Application Support/Firefox")
	end

	def profilesListing
		return Helpers.ls_output_for_path("~/Library/Application Support/Firefox/Profiles")
	end

	def extensionsFolderListingForProfile(path)
		return Helpers.ls_output_for_path(path)
	end


end

