require 'helpers'
require 'ostruct'

class OSX
	attr_reader :hostname
	attr_reader :version
	attr_reader :build
	attr_reader :system_profile
	attr_reader :user_group_summary
	attr_reader :username
	attr_reader :uptime
	attr_reader :relevant_processes
	attr_reader :available_disk_space
	attr_reader :user_is_admin
	attr_reader :installed_internet_plugins
	attr_reader :installed_scripting_additions
	attr_reader :default_browser
	attr_reader :sophos_installed
	attr_reader :sophos_web_protection_enabled
	attr_reader :sophos_localhost_allowed
	attr_reader :little_snitch_installed
	attr_reader :hands_off_installed
	attr_reader :intego_virus_barrier_installed
	attr_reader :avast_antivirus_installed
	attr_reader :avast_antivirus_running
	attr_reader :eset_installed
	attr_reader :eset_started
	attr_reader :scep_started
	attr_reader :mackeeper_installed
	attr_reader :cleanmymac_installed
	attr_reader :trusteer_rapport_installed
	attr_reader :trusteer_rapport_started
	attr_reader :glimmerblocker_installed
	attr_reader :glimmerblocker_started
	attr_reader :all_processes
	attr_reader :dropbox_running
	attr_reader :proxy_information
	attr_reader :osx_firewall_enabled
	attr_reader :bonjour_running
	attr_reader :discoveryd_running
	attr_reader :locale_output
	attr_reader :lsof_output
	attr_reader :x3watch_running
	attr_reader :crl_revocation_setting
	attr_reader :ocsp_revocation_setting

	def initialize
		@hostname = get_hostname
		@version = get_version
		@build = get_build
		@system_profile = get_system_profile
		@user_group_summary = get_user_group_summary
		@username = get_username
		@uptime = get_uptime

		@all_processes = get_all_processes
		@relevant_processes = get_relevant_processes
		@dropbox_running = is_dropbox_running?

		@available_disk_space = get_available_disk_space
		@user_is_admin = user_is_admin?

		@installed_internet_plugins = get_installed_internet_plugins
		@installed_scripting_additions = get_installed_scripting_additions

		@default_browser = get_default_browser

		@sophos_installed = sophos_installed?
		@sophos_web_protection_enabled = sophos_web_protection_enabled?
		@sophos_localhost_allowed = sophos_localhost_allowed?

		@little_snitch_installed = little_snitch_installed?
	
		@hands_off_installed = hands_off_installed?
		@intego_virus_barrier_installed = intego_virus_barrier_installed?

		@avast_antivirus_installed = avast_antivirus_installed?
		@avast_antivirus_running = avast_antivirus_running?

		@eset_installed = eset_installed?
		@eset_started = eset_started?

		@scep_started = scep_started?

		@mackeeper_installed = mackeeper_installed?
		@cleanmymac_installed = cleanmymac_installed?

		@trusteer_rapport_installed = trusteer_rapport_installed?
		@trusteer_rapport_started = trusteer_rapport_started?
		
		@glimmerblocker_installed = glimmerblocker_installed?
		@glimmerblocker_started = glimmerblocker_started?

		#@proxy_information = get_proxy_information
		
		@osx_firewall_enabled = osx_firewall_enabled?
		@bonjour_running = bonjour_running?
		@discoveryd_running = discoveryd_running?

		@locale_output = get_locale_output
		@lsof_output = get_lsof_output

		@x3watch_running = x3watch_running?

		@crl_revocation_setting = get_crl_revocation_setting
		@ocsp_revocation_setting = get_ocsp_revocation_setting
	end

	def get_hostname
		%x{hostname -s}.chomp!
	end

	def get_version
		sw_vers = %x{sw_vers}
		
		return $1 if sw_vers =~ /ProductVersion\:\s(.*?)\n/
		
		return "Unknown: #{sw_vers.inspect}"
	end

	def get_build
		sw_vers = %x{/usr/bin/sw_vers}

		return $1 if sw_vers =~ /BuildVersion\:\s(.*?)\n/
		return "Unknown: #{sw_vers.inspect}" 
	end

	def get_system_profile
		%x{/usr/sbin/system_profiler SPHardwareDataType}.gsub(/^\s*Serial Number.*$/, "").gsub(/^\s*Hardware UUID:.*$/, "")
	end

	def get_user_group_summary
		%x{/usr/bin/id}
	end

	def get_username
		%x{whoami}.chomp!
	end

	def get_uptime
		%x{uptime}.chomp!
	end

	def get_all_processes
		processes = %x{ps -Ajc}.split("\n").slice(1..-1).collect do |process|
			process_columns = process.split(" ", 10)
			
			current_process = OpenStruct.new
			current_process.user = process_columns[0]
			current_process.pid = process_columns[1]
			current_process.state = process_columns[6]
			current_process.command = process_columns[9]
			current_process
		end
	end

	def get_relevant_processes
		@all_processes.select do |process|
			process.command =~ %r{1P} || process.command =~ %r{onepassword} || process.command =~ %r{virus} || process.command =~ %r{Dropbox} || process.command =~ %r{Safari} || process.command =~ %r{irefox} || process.command =~ %r{Google\ Chrome} || process.command =~ %r{Opera}
		end
	end

	def search_for_process(query)
		search_regex = Regexp.new(query)
		@all_processes.select do |process|
			process.command =~ search_regex
		end
	end

	def is_dropbox_running?
		results = search_for_process("Dropbox")

		passed = false

		if results.count > 0
			results.each do |process|
				passed = true if process.user == @username
			end
		end

		return passed
	end

	def get_available_disk_space
		return %x{/bin/df -h}
	end

	def user_is_admin?
		id_output = %x{id}
    id_output =~ /uid=\d+\(([^\)]+)\)/
    shortname = $1
    if id_output =~ /groups=.*?admin/
			true
		else
			false
		end
	end

	def get_installed_internet_plugins
		results = Helpers.ls_output_for_path("/Library/Internet\ Plug-Ins/")
		results << "\n\n" << Helpers.ls_output_for_path("#{HOME}/Library/Internet\ Plug-Ins/")
	end

	def get_installed_scripting_additions
		results = Helpers.ls_output_for_path("/Library/ScriptingAdditions/")
		results << "\n\n" << Helpers.ls_output_for_path("#{HOME}/Library/ScriptingAdditions/")
	end

	def verify_running_process(pid)
		#pid = `ps ux | grep "#{search}" | grep -v grep | awk '{print $2}'`.chomp!
		result = %x{/usr/bin/codesign -vvvv #{pid} 2>&1}

		if result =~ /dynamically valid/
			true
		else
			false
		end
	end

	def get_default_browser
		command = "/usr/libexec/Plistbuddy -c "
		plist = "#{HOME}/Library/Preferences/com.apple.LaunchServices.plist"
		
		begin
			dict_count = `/usr/libexec/PlistBuddy -c "Print LSHandlers:" #{plist} | grep "Dict" | wc -l`.chomp!.strip!.to_i
		rescue NoMethodError
			return "Failed to retrieve default browser"
		end

		browser = "com.apple.safari"

		dict_count.times do |index|
				result = `#{command} 'Print LSHandlers:#{index}:LSHandlerURLScheme' #{plist} 2>&1`
				if result.chomp! == "http"
						browser = `#{command} 'Print LSHandlers:#{index}:LSHandlerRoleAll' #{plist}`.chomp!
				end
		end

		if browser == "com.apple.safari"
			return "Safari"
		elsif browser == "org.mozilla.firefox"
			return "Firefox"
		elsif browser == "com.google.chrome"
			return "Chrome"
		elsif browser == "com.operasoftware.operanext"
			return "Opera Next"
		elsif browser == "com.operasoftware.opera"
			return "Opera"			
		elsif browser == "com.google.chrome.canary"
			return "Chrome Canary"
		elsif browser == "org.chromium.chromium"
			return "Chromium"
		elsif browser ==  "com.choosyosx.choosy"
			return "Choosy"
		else
			return browser
		end
	end

	# Always check sophos_installed? because they leave this behind when uninstalled installed + this = true
	def sophos_web_protection_enabled?
		result = `/usr/libexec/PlistBuddy -c 'Print WebProtectionFilteringEnabled' /Library/Preferences/com.sophos.sav.plist`

		if result.chomp! == "true"
			return true
		else
			return false
		end
	end

	def sophos_installed?
		File.exists?("/Library/LaunchDaemons/com.sophos.scan.plist")
	end

	def sophos_localhost_allowed?
		result = %x{defaults read /Library/Preferences/com.sophos.sav.plist WebProtectionApprovedSites}

		if result =~ /127\.0\.0\.1/
			return true
		else
			return false
		end
	end

	def little_snitch_installed?
		File.exists?("/Library/LaunchDaemons/at.obdev.littlesnitchd.plist")
	end

	def hands_off_installed?
		File.exists?("/Library/LaunchDaemons/com.metakine.handsoff.daemon.plist")
	end

	def intego_virus_barrier_installed?
		# Glob used because they seem to change the name of these plists between releases
		result = Dir.glob("/Library/LaunchDaemons/com.intego*.plist")

		if result.count > 0 
			return true
		else
			return false
		end
	end

	def avast_antivirus_installed?
		File.exists?("/Library/LaunchDaemons/com.avast.init.plist")
	end

	def avast_antivirus_running?
		results = search_for_process("com.avast")

		if results.count > 0
			return true
		else
			return false
		end
	end


	def eset_installed?
		File.exists?("/Library/LaunchDaemons/com.eset.esets_daemon.plist")
	end

	def eset_started?
		results = search_for_process("esets_")

		if results.count > 0 
			return true
		else
			return false
		end
	end

	def scep_started?
		results = search_for_process("scep_daemon")

		if results.count > 0 
			return true
		else
			return false
		end
	end

	def mackeeper_installed?
		File.exists?("#{HOME}/Library/LaunchAgents/com.zeobit.MacKeeper.Helper.plist")
	end

	def cleanmymac_installed?
		result = Dir.glob("#{HOME}/Library/LaunchAgents/com.macpaw.CleanMyMac*.plist")

		if result.count > 0
			return true
		else
			return false
		end
	end

	def osx_firewall_enabled?
		enabled = Helpers.readDefaults("/Library/Preferences/com.apple.alf", "globalstate")

		if enabled.chomp! == "1"
			return true
		else
			return false
		end
	end

	def bonjour_running?
		results = search_for_process("mDNSResponder")	

		results.count > 0 ? true : false
	end

	def discoveryd_running?
		results = search_for_process("discoveryd")

		results.count > 0 ? true : false
	end

	def get_proxy_information
		list_of_services_raw = %x{networksetup -listallnetworkservices}
		split_list_of_services = list_of_services_raw.split("\n")[1..-1]

		# Remove any items that are disabled (they are prefixed with *)
		list_of_services = split_list_of_services.reject do |service|
			service =~ /\*/
		end

		return [] unless list_of_services.count >= 0

		proxy_info = []

		list_of_services.each do |service|
			temp_data = OpenStruct.new
			temp_data.service = service

			service_http_output = %x{networksetup -getwebproxy "#{service}"}.split("\n")

			service_http_output.each do |http|

				if http =~ /^Enabled: (.*)/
					if $1 == "No"
						temp_data.http_enabled = false
					else
						temp_data.http_enabled = true
					end
				end

				if http =~ /Server: (.*)/
					if $1 == ""
						temp_data.http_server = "No Server Set"
					else
						temp_data.http_server = $1
					end
				end

				if http =~ /Port: (.*)/
					temp_data.http_port = $1
				end

				if http =~ /^Authenticated Proxy Enabled: (.*)/
					if $1 == "0"
						temp_data.http_auth_proxy_enabled = false
					else
						temp_data.http_auth_proxy_enabled = true
					end
				end
			end

			service_https_output = %x{networksetup -getsecurewebproxy "#{service}"}.split("\n")

			service_https_output.each do |https|
				if https =~ /^Enabled: (.*)/
					if $1 == "No"
						temp_data.https_enabled = false
					else
						temp_data.https_enabled = true
					end
				end

				if https =~ /Server: (.*)/
					if $1 == ""
						temp_data.https_server = "No Server Set"
					else
						temp_data.https_server = $1
					end
				end

				if https =~ /Port: (.*)/
					temp_data.https_port = $1
				end

				if https =~ /^Authenticated Proxy Enabled: (.*)/
					if $1 == "0"
						temp_data.https_auth_proxy_enabled = false
					else
						temp_data.https_auth_proxy_enabled = true
					end
				end
			end

			service_bypass_domains_output = %x{networksetup -getproxybypassdomains "#{service}"}.split("\n")
			if service_bypass_domains_output[0] =~ /There aren't any bypass domains set/
				temp_data.proxy_bypass_domains = ["No bypass domains found"]
			else
				temp_data.proxy_bypass_domains = service_bypass_domains_output
			end

			service_proxy_auto_discovery_output = %x{networksetup -getproxyautodiscovery "#{service}"}.split("\n")
			if service_proxy_auto_discovery_output[0] =~ /Auto Proxy Discovery: (.*)/
				if $1 == "Off"
					temp_data.proxy_auto_discovery = false
				else
					temp_data.proxy_auto_discovery = true
				end
			else
				temp_data.proxy_auto_discovery = false
			end

			proxy_info.push temp_data

		end

		return proxy_info
			
	end

	def trusteer_rapport_installed?
		File.exist?("/Library/Rapport/bin/rapportd")
	end

	def trusteer_rapport_started?
		%x{launchctl list | grep com.trusteer.rapport.rapportd}.chomp != ""
	end

	def glimmerblocker_installed?
		File.exist?("/Library/LaunchDaemons/org.glimmerblocker.proxy.plist")
	end

	def glimmerblocker_started?
		results = search_for_process("GlimmerBlockerProxy")

		passed = false

		if results.count > 0
			passed = true
		end

		return passed
	end

	def get_locale_output
		%x{locale}
	end

	def get_lsof_output
		%x{/usr/sbin/lsof -iTCP@127.0.01}
	end

	def x3watch_running?
		results = search_for_process("x3d")

		return true if results.count > 0
		return false
	end

	def get_crl_revocation_setting
		results = Helpers.readDefaults("com.apple.security.revocation", "CRLStyle")

		case results.chomp!
		when "RequireForAll"
			"Require For All Certificates"
		when "RequireIfPresent"
			"Require If Certificate Indicates"
		when "None"
			"Off"
		else
			"Best Attempt"
		end
	end

	def get_ocsp_revocation_setting
		results = Helpers.readDefaults("com.apple.security.revocation", "OCSPStyle")

		case results.chomp!
		when "RequireForAll"
			"Require For All Certificates"
		when "RequireIfPresent"
			"Require If Certificate Indicates"
		when "None"
			"Off"
		else
			"Best Attempt"
		end
	end
end
