require 'helpers'

class Safari
	attr_reader :version
	attr_reader :auto_fill_from_address_book
	attr_reader :auto_fill_miscellaneous_forms
	attr_reader :auto_fill_passwords
	attr_reader :javascript_enabled
	attr_reader :private_browsing_enabled
	attr_reader :develop_menu_enabled
	attr_reader :install_extension_updates_automatically
	attr_reader :installed_extensions
	attr_reader :extensions_folder_listing
	attr_reader :databases_folder_listing
	attr_reader :valid_code_signature
	attr_reader :code_signature_output
	attr_reader :jse3
	attr_reader :jse4_extension_settings_output

	def initialize
		@version = get_version
		@auto_fill_from_address_book = auto_fill_from_address_book?
		@auto_fill_miscellaneous_forms = auto_fill_miscellaneous_forms?
		@auto_fill_passwords = auto_fill_passwords?
		@javascript_enabled = javascript_enabled?
		@private_browsing_enabled = private_browsing_enabled?
		@develop_menu_enabled = develop_menu_enabled?
		@install_extension_updates_automatically = install_extension_updates_automatically?
		@installed_extensions = get_installed_extensions
		@extensions_folder_listing = get_extensions_folder_listing
		@databases_folder_listing = get_databases_folder_listing

		@valid_code_signature = Helpers.verify_code_signature("/Applications/Safari.app/")
		@code_signature_output = Helpers.code_signature_output("/Applications/Safari.app/")

		@jse3 = get_jse3_database_info
		@jse4_extension_settings_output = get_jse4_extension_settings_output
	end

	def get_version
		version = Helpers.readPlistValueForKeyPath("/Applications/Safari.app/Contents/Info.plist", "CFBundleShortVersionString")
		return version
	end

	def auto_fill_from_address_book?
		result = Helpers.readDefaults("com.apple.Safari", "AutoFillFromAddressBook")
		
		if result.chomp! == "0"
			return false
		else
			return true
		end
	end

	def auto_fill_miscellaneous_forms?
		result = Helpers.readDefaults("com.apple.Safari", "AutoFillMiscellaneousForms")

		if result.chomp! == "0"
			return false
		else
			return true
		end	
	end

	def auto_fill_passwords?
		result = Helpers.readDefaults("com.apple.Safari", "AutoFillPasswords")

		if result.chomp! == "0"
			return false
		else
			return true
		end
	end

	def javascript_enabled?
		result = Helpers.readDefaults("com.apple.Safari", "WebKitJavaScriptEnabled")

		if result.chomp! == "0"
			return false 
		else
			return true
		end
	end

	def install_extension_updates_automatically?
		result = Helpers.readDefaults("com.apple.Safari", "InstallExtensionUpdatesAutomatically")

		# By default this option is off in 10.8
		if Helpers.osx_version =~ /10\.8/
			if result.chomp! == "1"
				return true
			else
				return false
			end
		end

		# By default this option is on in 10.9
		if result.chomp! == "0"
			return false 
		else
			return true
		end
	end

	# private_browsing_enabled?
	#		Arguments: None
	#		Returns BOOL
	#
	def private_browsing_enabled?
		result = Helpers.readDefaults("com.apple.Safari", "WebKitPrivateBrowsingEnabled")

		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	# develop_menu_enabled?
	#		Arguments: None
	#		Returns: BOOL
	def develop_menu_enabled?
		result = Helpers.readDefaults("com.apple.Safari", "IncludeDevelopMenu")

		if result.chomp! == "1"
			return true
		else
			return false
		end
	end

	# installedExtensions
	# Arguments: None
	# Returns: Array of hashes - name/author/version
	#
	def get_installed_extensions_old
		# Unfortunately, this implementation does not detect if an extension is disabled in Safari
		installed_extensions = []

		# Users home folder
		home_folder = File.expand_path("~/")

		return installed_extensions unless File.exists?("#{home_folder}/Library/Caches/com.apple.Safari/Extensions/")
		
		# Get list of extensions in Cache folder
		extensions_folder = Dir.new("#{home_folder}/Library/Caches/com.apple.Safari/Extensions/")

		extensions_list = extensions_folder.reject { |folder| folder == "." || folder == ".." || folder == ".DS_Store" }

		extensions_list.each do |folder|
			current_directory = "#{home_folder}/Library/Caches/com.apple.Safari/Extensions/#{folder}"
			plist = "#{current_directory}/Info.plist"
			if File.exists?(plist)
				current_extension = {}

				current_extension["name"] = Helpers.findPlistValueForKey(plist, "CFBundleDisplayName")
				current_extension["author"] = Helpers.findPlistValueForKey(plist, "Author")
				current_extension["version"] = Helpers.findPlistValueForKey(plist, "CFBundleShortVersionString")

				installed_extensions.push current_extension
			end
		end
	
		return installed_extensions	
	end

	def get_installed_extensions
		installed_extensions = []
		home_folder = File.expand_path("~/")
		extensions_plist = "#{home_folder}/Library/Safari/Extensions/Extensions.plist"
		command = "/usr/libexec/PlistBuddy -c"

		return installed_extensions unless File.exists?(extensions_plist)

		total_extensions_raw = %x{#{command} 'Print "Installed Extensions"' #{extensions_plist} | grep "Dict" | wc -l}

		total_extensions = total_extensions_raw.gsub(/\s+/, "").to_i

		total_extensions.times do |index|
			current_extension = OpenStruct.new
			bundle_name = %x{#{command} 'Print "Installed Extensions":#{index}:"Bundle Directory Name"' #{extensions_plist}}
			current_extension.bundle_directory_name = bundle_name.chomp!
			enabled_raw = %x{#{command} 'Print "Installed Extensions":#{index}:"Enabled"' #{extensions_plist}}

			if enabled_raw =~ /true/
				current_extension.enabled = true
			else
				current_extension.enabled = false
			end

			current_directory = "#{home_folder}/Library/Caches/com.apple.Safari/Extensions/#{current_extension.bundle_directory_name}"
			plist = "#{current_directory}/Info.plist"
			if File.exists?(plist)
				current_extension.extname = Helpers.findPlistValueForKey(plist, "CFBundleDisplayName")
				current_extension.author = Helpers.findPlistValueForKey(plist, "Author")
				current_extension.version = Helpers.findPlistValueForKey(plist, "CFBundleShortVersionString")
			else
				current_extension.extname = "N/A"
				current_extension.author = "N/A"
				current_extension.version = "N/A"
			end

			installed_extensions << current_extension 
		end

		return installed_extensions
	end

	def get_extensions_folder_listing
		home_folder = Helpers.home

		return Helpers.ls_output_for_path("#{home_folder}/Library/Safari/Extensions")

	end

	def get_databases_folder_listing
		home_folder = Helpers.home 

		return Helpers.ls_output_for_path("#{home_folder}/Library/Safari/Databases")
	end

	def get_jse3_database_info
		# Check whether all Files exist!
		safari_db_path = "#{HOME}/Library/Safari/Databases/Databases.db"		
		full_jse3_db_folder = Dir.glob("#{HOME}/Library/Safari/Databases/safari-extension_com.agilebits.onepassword-safari-2bua8c4s2c_*/")[0] rescue nil

		jse3_info = OpenStruct.new

		if File.exists?(safari_db_path) && full_jse3_db_folder != nil
			jse3_folder = full_jse3_db_folder.split("/")[-1]			
			raw_jse3_db_filename = Helpers.sqlite_query(safari_db_path, "SELECT path FROM Databases WHERE origin = '#{jse3_folder}'")
			jse3_db_filename = raw_jse3_db_filename.chomp!
		else
			jse3_db_filename = ""
		end

		full_db_path = "#{full_jse3_db_folder}/#{jse3_db_filename}"

		if File.exists?(full_db_path) && full_jse3_db_folder != nil
			raw_total_items = Helpers.sqlite_query(full_db_path, "SELECT count(*) FROM items;")
			jse3_info.total_items = raw_total_items.chomp!.to_i

			raw_total_keys = Helpers.sqlite_query(full_db_path, "SELECT count(*) FROM keys;")
			jse3_info.total_keys = raw_total_keys.chomp!.to_i

			raw_unsynced_items = Helpers.sqlite_query(full_db_path, "SELECT count(*) FROM items WHERE contents_hash = '777';")
			jse3_info.unsynced_items = raw_unsynced_items.chomp!.to_i

			jse3_info.config_info = Helpers.sqlite_query(full_db_path, "SELECT * FROM config;")

			jse3_info.schema = Helpers.sqlite_query(full_db_path, ".schema")
		else
			jse3_info.total_items = 0 
			jse3_info.total_keys = 0
			jse3_info.unsynced_items = 0
			jse3_info.config_info = "DB Does Not Exist"
			jse3_info.schema = "DB Does Not Exist"
		end

		return jse3_info
				
	end

	def get_jse4_extension_settings_output
		if Helpers.osx_version =~ /10\.10/
			Helpers.readDefaults("com.apple.Safari.Extensions", "ExtensionSettings-com.agilebits.onepassword4-safari-2BUA8C4S2C")
		else
			Helpers.readDefaults("com.apple.Safari", "ExtensionSettings-com.agilebits.onepassword4-safari-2BUA8C4S2C")
		end
	end
end
